#include "unblock.h"

static void check_chrominance_(DoubleImage *di, char *chname);

static void unblock_(DoubleImage **p_this, DoubleImage *orig);

static void unblock_(DoubleImage **p_this, DoubleImage *other);

static void unblock_h_(
    DoubleImage **p_this,
    DoubleImage *orig,
    DoubleImage *in);

static void unblock_v_(
    DoubleImage **p_this,
    DoubleImage *orig,
    DoubleImage *in);

static void discrepancies_(int *b, int o, int *u, int *v);

static void check_(int i);

static void check2_(int i);

static short div_(int num, ushort den);

static short limit_(int i);

static void adjustments_(ulong *ms, ulong *rs, byte *a);

static int adjust_(int s, byte *a);

static void correct_(int *b, int u, int v);

static void c_(int *v, int d);


void byte_image_new_unblock_jpeg(ByteImage **p_this, ByteImage *other) {
  byte num_channels;
  ulong half_width;
  ulong half_height;
  ulong width;
  ulong height;
  DoubleImage *cb;
  DoubleImage *_cb_halved = null;
  DoubleImage *_cb_halved_unblocked = null;
  DoubleImage *cr;
  DoubleImage *_cr_halved = null;
  DoubleImage *_cr_halved_unblocked = null;
  DoubleImage *_di_other = null;
  DoubleImage *_di_out = null;
  DoubleImage *y;
  DoubleImage *_y_unblocked = null;
  DoubleImage **_dis = null;

  width = other->width;
  height = other->height;

  double_image_new_from_byte(&_di_other, other);
  num_channels = double_images_new_split_channels(&_dis, _di_other);
  double_image_delete(&_di_other);

  if (num_channels != 3) {
    throw("Num channels (%u) should be 3", num_channels);
  }

  y = _dis[0];
  cb = _dis[1];
  cr = _dis[2];

  check_chrominance_(cb, "Cb");
  check_chrominance_(cr, "Cr");

  half_width = (width + 1) / 2;
  half_height = (height + 1) / 2;

  double_image_new_resize(
      &_cb_halved,
      cb,
      half_width,
      half_height,
      0.5,
      MAGIC_RESIZER_NEAREST_NEIGHBOR,
      0,
      0);
  double_image_new_resize(
      &_cr_halved,
      cr,
      half_width,
      half_height,
      0.5,
      MAGIC_RESIZER_NEAREST_NEIGHBOR,
      0,
      0);

  unblock_(&_y_unblocked, y);
  unblock_(&_cb_halved_unblocked, _cb_halved);
  unblock_(&_cr_halved_unblocked, _cr_halved);

  double_image_delete(&_cr_halved);
  double_image_delete(&_cb_halved);
  delete(&_dis, num_channels, double_image_delete);

  new(&_dis, 3);
  donate(&_y_unblocked, &_dis[0]);

  double_image_new_resize(
      &_dis[1],
      _cb_halved_unblocked,
      width,
      height,
      2,
      MAGIC_RESIZER_MAGIC_KERNEL_SHARP,
      6,
      7);
  double_image_new_resize(
      &_dis[2],
      _cr_halved_unblocked,
      width,
      height,
      2,
      MAGIC_RESIZER_MAGIC_KERNEL_SHARP,
      6,
      7);

  double_image_delete(&_cr_halved_unblocked);
  double_image_delete(&_cb_halved_unblocked);

  double_image_new_from_channels(&_di_out, _dis, num_channels);
  delete(&_dis, num_channels, double_image_delete);

  byte_image_new_from_double(p_this, _di_out);
  double_image_delete(&_di_out);
}


static void check_chrominance_(DoubleImage *di, char *chname) {
  ulong i;
  ulong j;
  double d;
  double e;
  double **channel;

  if (di->numChannels != 1 ) {
    throw("impossible");
  }

  channel = di->_channels[0];

  for (i = 0; i < di->width; i++) {
    for (j = 0; j < di->height; j++) {
      if ((d = channel[i][j]) != (e = channel[i / 2 * 2][j / 2 * 2])) {
        throw("Chrominance channel %s not downsampled without "
            "fancy_upsampling: %.16g v %.16g at (%lu, %lu)",
            chname,
            d,
            e,
            i,
            j);
      }
    }
  }
}

static void unblock_(DoubleImage **p_this, DoubleImage *orig) {
  ulong i;
  ulong j;
  double **this_c;
  double **hv_c;
  double **vh_c;
  DoubleImage *this;
  DoubleImage *_temp = null;
  DoubleImage *_hv = null;
  DoubleImage *_vh = null;

  if (!orig->gamma) {
    throw("Can only apply the UnBlock algorithm in gamma (YCbCr) space");
  }

  unblock_h_(&_temp, orig, orig);
  unblock_v_(&_hv, orig, _temp);
  double_image_delete(&_temp);

  unblock_v_(&_temp, orig, orig);
  unblock_h_(&_vh, orig, _temp);
  double_image_delete(&_temp);

  double_image_new_y(p_this, orig->width, orig->height);
  this = *p_this;

  this->gamma = true;

  this_c = this->_channels[0];
  hv_c = _hv->_channels[0];
  vh_c = _vh->_channels[0];

  for (i = 0; i < this->width; i++) {
    for (j = 0; j < this->height; j++) {
      this_c[i][j] = 0.5 * (hv_c[i][j] + vh_c[i][j]);
    }
  }

  double_image_delete(&_hv);
  double_image_delete(&_vh);
}

static void unblock_h_(
    DoubleImage **p_this,
    DoubleImage *orig,
    DoubleImage *in) {
  DoubleImage *_orig_t = null;
  DoubleImage *_in_t = null;
  DoubleImage *_out_t = null;

  double_image_new_transpose(&_orig_t, orig);
  double_image_new_transpose(&_in_t, in);
  unblock_v_(&_out_t, _orig_t, _in_t);
  double_image_delete(&_in_t);
  double_image_delete(&_orig_t);
  double_image_new_transpose(p_this, _out_t);
  double_image_delete(&_out_t);
}

static void unblock_v_(
    DoubleImage **p_this,
    DoubleImage *orig,
    DoubleImage *in) {
  byte m;
  byte *_aus = null;
  byte *_avs = null;
  int bu;
  int bv;
  int iu;
  int iv;
  int u;
  int v;
  int b[16];
  ulong i;
  ulong j;
  ulong r;
  ulong *_bus = null;
  ulong *_bvs = null;
  ulong *_ius = null;
  ulong *_ivs = null;
  DoubleImage *this;

  if (orig->numChannels != 1) {
    throw("impossible (%u)", orig->numChannels);
  }

  if (in->numChannels != 1) {
    throw("impossible (%u)", in->numChannels);
  }

  double_image_new_copy(p_this, in);
  this = *p_this;

  if (in->width < 9) {
    return;
  }

  new(&_bus, 256);
  new(&_bvs, 256);
  new(&_ius, 256);
  new(&_ivs, 256);
  new(&_aus, 256);
  new(&_avs, 256);

  for (j = 0; j < orig->height; j++) {
    for (m = 8, i = 1; m < 10; m++, i++) {
      b[m] = (int)byte_bound(orig->_channels[0][i][j] * 255);
    }

    for (r = 8; r < orig->width; r += 8) {
      for (m = 0; m < 2; m++) {
        b[m] = b[m + 8];
      }

      for (i = r - 5; m < 10; i++, m++) {
        b[m] = i < orig->width
          ? (int)byte_bound(orig->_channels[0][i][j] * 255)
          : -1;
      }

      discrepancies_(b, 0, &iu, &iv);
      _ius[abs(iu)]++;
      _ivs[abs(iv)]++;
      discrepancies_(b, 4, &bu, &bv);
      _bus[abs(bu)]++;
      _bvs[abs(bv)]++;
    }
  }

  adjustments_(_bus, _ius, _aus);
  adjustments_(_bvs, _ivs, _avs);

  for (j = 0; j < in->height; j++) {
    for (m = 8, i = 0; m < 16; m++, i++) {
      b[m] = (int)byte_bound(in->_channels[0][i][j] * 255);
    }

    for (r = 8; r < in->width; r += 8) {
      for (m = 0; m < 8; m++) {
        b[m] = b[m + 8];
      }

      for (i = r; m < 16; i++, m++) {
        b[m] = i < orig->width
          ? (int)byte_bound(in->_channels[0][i][j] * 255)
          : -1;
      }

      discrepancies_(b, 5, &u, &v);
      u = adjust_(u, _aus);
      v = adjust_(v, _avs);
      correct_(b, u, v);

      for (m = 0, i = r - 8; m < 16 && i < this->width; m++, i++) {
        if (b[m] < 0 || b[m] > 255) {
          throw("woof");
        }

        this->_channels[0][i][j] = b[m] / 255.0;
      }
    }

    for (; m < 16 && i < this->width; m++, i++) {
      this->_channels[0][i][j] = b[m] / 255.0;
    }
  }

  deletev(&_avs);
  deletev(&_aus);
  deletev(&_ivs);
  deletev(&_ius);
  deletev(&_bvs);
  deletev(&_bus);
}

static void discrepancies_(int *b, int o, int *u, int *v) {
  int i6;
  int i7;
  int i8;
  int i9;
  int i10;
  int i11;
  int uu;
  int vv;

  check2_(i6 = b[o++]);
  check2_(i7 = b[o++]);
  check2_(i8 = b[o++]);
  check2_(i9 = b[o++]);
  check_(i10 = b[o++]);
  check_(i11 = b[o]);

  if (i11 >= 0) {
    check2_(i10);
    uu = div_(15 * (i9 - i8) - 10 * (i10 - i7) + 3 * (i11 - i6), 8);
    vv = -2 * (i9 + i8) + 3 * (i10 + i7) - (i11 + i6);
  } else if (i10 >= 0) {
    uu = div_(-3 * i6 + 10 * i7 - 15 * i8 + 12 * i9 - 4 * i10, 8);
    vv = -i6 + 3 * i7 - 2 * i8 - i9 + i10;
  } else {
    uu = div_(-3 * i6 + 10 * i7 - 15 * i8 + 8 * i9, 8);
    vv = 0;
  }

  *u = limit_(uu);
  *v = limit_(vv);
}

static void check_(int i) {
  if (i < -1 || i > +255) {
    throw("i (%d) invalid", i);
  }
}

static void check2_(int i) {
  if (i < 0 || i > +255) {
    throw("i (%d) invalid", i);
  }
}

static short div_(int num, ushort den) {
  bool neg;
  int r;

  neg = num < 0;

  if (neg) {
    num = -num;
  }

  r = (num + (int)den / 2) / (int)den;

  if (neg) {
    r = -r;
  }

  return r;
}

static short limit_(int i) {
  return i < -255 ? -255 : i > +255 ? +255 : (short)i;
}

static void adjustments_(ulong *ms, ulong *rs, byte *a) {
  ushort m;
  ushort r;
  ulong cm;
  ulong cr;

  for (m = 0, r = 0, cm = 0, cr = 0; m < 256; m++) {
    cm += ms[m];

    while (cr < cm) {
      cr += rs[r++];
    }

    if (cr > cm) {
      cr -= rs[--r];
    }

    a[m] = r == 0 ? (byte)m : m > r ? (byte)(m + 1 - r) : (byte)0;
  }
}

static int adjust_(int k, byte *ks) {
  return k < 0 ? -ks[-k] : ks[k];
}

static void correct_(int *b, int u, int v) {
  int d2u;
  int d3u;
  int d4u;
  int d5u;
  int d6u;
  int d7u;
  int d8u;
  int d1v;
  int d2v;
  int d3v;
  int d4v;
  int d5v;
  int d6v;
  int d7v;
  int d8v;

  if (u == 0 && v == 0) {
    return;
  }

  d2u = div_(u, 64);
  d3u = div_(3 * u, 64);
  d4u = div_(3 * u, 32);
  d5u = div_(5 * u, 32);
  d6u = div_(15 * u, 64);
  d7u = div_(21 * u , 64);
  d8u = div_(7 * u, 16);

  d1v = div_(-v, 256);
  d2v = div_(-11 * v, 256);
  d3v = div_(-31 * v, 256);
  d4v = div_(-58 * v, 256);
  d5v = div_(-57 * v, 256);
  d6v = div_(-22 * v, 256);
  d7v = div_(+42 * v, 256);
  d8v = div_(+138 * v, 256);

  c_(&b[0], d1v);
  c_(&b[1], d2u + d2v);
  c_(&b[2], d3u + d3v);
  c_(&b[3], d4u + d4v);
  c_(&b[4], d5u + d5v);
  c_(&b[5], d6u + d6v);
  c_(&b[6], d7u + d7v);
  c_(&b[7], d8u + d8v);
  c_(&b[8], d8v - d8u);
  c_(&b[9], d7v - d7u);
  c_(&b[10], d6v - d6u);
  c_(&b[11], d5v - d5u);
  c_(&b[12], d4v - d4u);
  c_(&b[13], d3v - d3u);
  c_(&b[14], d2v - d2u);
  c_(&b[15], d1v);
}

static void c_(int *v, int d) {
  *v += d;

  if (*v < 0) {
    *v = 0;
  } else if (*v > +255) {
    *v = +255;
  }
}
