#include "unblock_jpeg.h"

int main(int argc, char *argv[]) {
  byte quality;
  char *filename;
  ByteImage *_in = null;
  ByteImage *_out = null;

  args_description(
      "Apply the UnBlock algorithm to a JPEG image, writing out the result as "
      "either JPEG or PNG");
  args_string("input-filename", "Input JPEG image");
  args_string("output-filename", "Output UnBlocked image");
  args_flag("png", 'p', "Save the output file in PNG format");
  args_flag("jpeg", 'j', "Save the output file in JPEG format");
  args_optional_integer(
      "jpeg-quality",
      'q',
      jpeg_file_get_default_quality(),
      "The JPEG quality to use for the output JPEG file, between 0 and 100");
  args_parse(argc, argv);

  byte_image_new_load_jpeg_ycc(&_in, args_get_string("input-filename"));
  byte_image_new_unblock_jpeg(&_out, _in);
  byte_image_delete(&_in);

  filename = args_get_string("output-filename");
  quality = args_get_bounded_integer("jpeg-quality", 0, 100);

  if (args_get_flag_veto("png", "jpeg")) {
    byte_image_save_png(_out, filename);
  } else if (args_get_flag("jpeg")) {
    byte_image_save_jpeg(_out, filename, quality);
  } else {
    byte_image_save_quality(_out, filename, quality);
  }

  byte_image_delete(&_out);

  return 0;
}
