#include "date_test_death.h"

START_DEATH

TEST(Date, Validate) {
  EXPECT_DEATH(
      date_validate("woof"),
      "Could not parse 'woof' as a 'YYYY-MM-DD' date string");
  EXPECT_DEATH(
      date_validate("19660924"),
      "Could not parse '19,660,924' as a 'YYYY-MM-DD' date string");
  EXPECT_DEATH(
      date_validate("1966-09"),
      "Could not parse '1966-09' as a 'YYYY-MM-DD' date string");
  EXPECT_DEATH(
      date_validate("1966-09-24 "),
      "Could not parse '1966-09-24 ' as a 'YYYY-MM-DD' date string");

  /* Library only handles dates from 1901.
   */
  EXPECT_DEATH(
      date_validate("1900-01-01"),
      "Could not parse '1900-01-01' as a 'YYYY-MM-DD' date string");

  /* Library only handles dates to 2079.
   */
  EXPECT_DEATH(
      date_validate("2080-01-01"),
      "Could not parse '2080-01-01' as a 'YYYY-MM-DD' date string");
}

TEST(Date, MonthShortnameFromValue) {
  EXPECT_DEATH(
      date_month_shortname_from_value(0),
      "Invalid month value 0: must be in [1, 12]");
  EXPECT_DEATH(
      date_month_shortname_from_value(13),
      "Invalid month value 13: must be in [1, 12]");
  EXPECT_DEATH(
      date_month_shortname_from_value(255),
      "Invalid month value 255: must be in [1, 12]");
}

TEST(Date, MonthLongnameFromValue) {
  EXPECT_DEATH(
      date_month_longname_from_value(0),
      "Invalid month value 0: must be in [1, 12]");
  EXPECT_DEATH(
      date_month_longname_from_value(13),
      "Invalid month value 13: must be in [1, 12]");
  EXPECT_DEATH(
      date_month_longname_from_value(255),
      "Invalid month value 255: must be in [1, 12]");
}

TEST(Date, DayOfWeekShortnameFromIndex) {
  EXPECT_DEATH(
      date_day_of_week_shortname_from_index(7),
      "Invalid day of week index 7: must be in [0, 6]");
  EXPECT_DEATH(
      date_day_of_week_shortname_from_index(255),
      "Invalid day of week index 255: must be in [0, 6]");
}

TEST(Date, DayOfWeekLongnameFromIndex) {
  EXPECT_DEATH(
      date_day_of_week_longname_from_index(7),
      "Invalid day of week index 7: must be in [0, 6]");
  EXPECT_DEATH(
      date_day_of_week_longname_from_index(255),
      "Invalid day of week index 255: must be in [0, 6]");
}

FINISH_DEATH
